// Copyright (c) 2012 Albrecht Weinert, Bochum
// common.h 2 2017-01-25 16:49:03Z albrecht

/**
 * \addtogroup weAutSys
 * @{
 * \addtogroup appLaySup
 * @{
 * \addtogroup comTypes
 * @{
 *
 * \file common.h
 *
 *  \ref intro_sec "weAutSys"' common types and helper functions
 *
 *  This file contains the definitions for \ref intro_sec "weAutSys"'
 *  common types and helper macros or functions. They are utilised in more
 *  than one application field respectively application or system module
 *  &mdash but not in bootloaders.
 *
 * This file is part of \ref intro_sec "weAutSys" &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2012  Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp;
 *                            <<a href="http://a-weinert.de">a-weinert.de</a>>
 * 2
 * 25.01.2017
 */
//  SVN last change by A. Weinert

/** @} */ // Doxygen addtogroup   comTypes
/** @} */ // Doxygen addtogroup appLaySup

#ifndef COMMON_H
#define COMMON_H
#include <stdint.h>
#include <stdio.h>
#include "pt/pt.h"
#include "arch/config.h"
#include "we-aut_sys/ll_common.h"

/**
 * \addtogroup appLaySup
 * @{
 * \addtogroup comTypes
 * @{
 */


//--------------- common texts ----------------------------------------

/** A short string with just one blank and four linefeeds in flash memory
 *
 *  This flash string as all cognates (\ref  bLF1, \ref bLF2 and many more)
 *  are 0 terminated. As flash strings they are immutable (by all normal
 *  and erroneous program means). They should be used whenever their content
 *  is asked for to save precious RAM.
 */
extern char const l4nefeeds[];
extern char const bLF1[]; //!< 1 blank, 1 linefeed 0 terminated in flash memory
extern char const bLF2[]; //!< 1 blank, 2 linefeed 0 terminated in flash memory


//------- threading and threading functions predefinitions and constants ---

/** Type of a protothread function (void)
 *
 *  \c funV_t is a function returning \ref PT_WAITING, \ref PT_YIELDED,
 *  \ref PT_EXITED or \ref PT_ENDED  and taking no parameter.
 */
typedef ptfnct_t (funV_t)(void);

/** Pointer to a protothread function (void)
 *
 *  \c p2ptFunV is a pointer to a function returning \ref PT_WAITING,
 *  \ref PT_YIELDED, \ref PT_EXITED or \ref PT_ENDED and no parameter.
 */
typedef funV_t * p2ptFunV;

struct mThr_data_t; // the minimum (small) structure type -------------

/** Type of a protothread function (struct mThr_data_t *)
 *
 *  \c funU_t is a function returning \ref PT_WAITING, \ref PT_YIELDED,
 *  \ref PT_EXITED or \ref PT_ENDED and taking a pointer
 *  to the basic minimal thread state (\ref mThr_data_t) as parameter.
 */
typedef ptfnct_t (funU_t)(struct mThr_data_t *uthr_data);

/** Pointer to a protothread function (struct mThr_data_t *)
 *
 *  \c p2ptFun is a pointer to a function returning\ref PT_WAITING,
 *  \ref PT_YIELDED, \ref PT_EXITED or \ref PT_ENDED  and taking a pointer
 *  to the basic minimal thread state  (\ref mThr_data_t)
 *  as parameter.
 */
typedef funU_t * p2ptFun;

struct thr_data_t; // the common (multi-use) structure type ------------

/** Type of a protothread function (struct thr_data_t *)
 *
 *  \c fun_t is a function returning \ref PT_WAITING, \ref PT_YIELDED,
 *  \ref PT_EXITED or \ref PT_ENDED  and taking a pointer
 *  to the common variable type thread state as parameter.
 *
 */
typedef ptfnct_t (fun_t)(struct thr_data_t * thrData);

/** Pointer to a protothread function (struct thr_data_t *)
 *
 *  \c p2ptFunC is a pointer to a function returning \ref PT_WAITING,
 *  \ref PT_YIELDED, \ref PT_EXITED or \ref PT_ENDED  and taking a pointer
 *  to the common variable type thread state (\ref thr_data_t)
 *  as parameter.
 */
typedef fun_t * p2ptFunC;


/** @} */ // Doxygen addtogroup comTypes
/**
* \addtogroup modbusServer
* @{
*/

struct modThr_data_t; // the Modbus handler (thread) structure type ---------

/** Type of a protothread function (modThr_data_t *)
 *
 *  \c funM_t is a function returning \ref PT_WAITING, \ref PT_YIELDED,
 *  \ref PT_EXITED or \ref PT_ENDED  and taking a pointer to a
 *  Modbus handler (thread) state (\ref modThr_data_t) as parameter.
 */
typedef ptfnct_t (funM_t)(struct modThr_data_t * thrData);

/** Pointer to a protothread function (modThr_data_t *)
 *
 *  \c p2ptFunM is a pointer to a function returning \ref PT_WAITING,
 *  \ref PT_YIELDED, \ref PT_EXITED or \ref PT_ENDED  and taking a pointer
 *  to  a Modbus handler (thread) state (\ref modThr_data_t) as parameter.
 */
typedef funM_t * p2ptFunM;

/** @} */ // Doxygen addtogroup modbusServer
/**
* \addtogroup comTypes
* @{
*/


struct cliThr_data_t; // the CLI (thread) structure type ------------

/** Type of a protothread function (cliThr_data_t *)
 *
 *  \c funA_t is a function returning \ref PT_WAITING, \ref PT_YIELDED,
 *  \ref PT_EXITED or \ref PT_ENDED  and taking a pointer
 *  to  a CLI (thread) state (\ref cliThr_data_t) as parameter.
 */
typedef ptfnct_t (funA_t)(struct cliThr_data_t * thrData);

/** Pointer to a protothread function (cliThr_data_t *)
 *
 *  \c p2ptFunA is a pointer to a function returning \ref PT_WAITING,
 *  \ref PT_YIELDED, \ref PT_EXITED or \ref PT_ENDED  and taking a pointer
 *  to a CLI (thread) state (\ref cliThr_data_t) as parameter.
 */
typedef funA_t * p2ptFunA;


/** The maximum number of characters in cliThr_data_t.line */
#define LEN_OF_CLITHR_LINE 128

/** Size of (extra) thread state data
 *
 * \showinitializer
 */
#define EXTRA_THR_ST_SZ 158
// at present (Sept 2012) the best / minimal value is LEN_OF_CLITHR_LINE + 30

/** The size of the biggest application state structure used in bytes
 *
 *  The type of the application state that is to be stored in the
 *  uip_conn structure may vary. This number shall be the maximum size
 *  of all application states.
 *
 *  \note This would have to be about 2 Kbyte as soon as A.D.'s HTTP
 *  implementation is used. This (one of the weakest parts of uIP) will
 *  can not be handled this way for small embedded systems.
 *
 *  \sa uip_conn
 *  \showinitializer
 */
#define SIZE_OF_BIGGEST_APPSTATE 164
// at present (April 2012) the best / minimal value is EXTRA_THR_ST_SZ + 6



//------- threading (special) structures      --------------------------------

/** The organisational data for a flash strings array output thread
 *
 *  \sa outFlashTextThreadF
 *  \sa initOutFlashTextThread
 */
typedef struct {
   pt_t pt; //!< The (raw) protothread data structure

/** The number of (flash) strings to be output
 *
 *  If the original (\ref initOutFlashTextThread "initialised") value is
 *  1..127 that is the number of 0-terminated flash strings to be output
 *  as is (i.e. no return appended) the value will be decremented to 0
 *  in the process if no NULL-pointer in the flash array of flash strings
 *  terminates it beforehand.
 *
 *  If the value was \ref initOutFlashTextThread "initialised" to -1 this
 *  will act as 1 outputting the 0-terminated flash string pointed to. But
 *  additionally, if the bytes after the terminating \\0 are \\x03 \\x01 \\x02
 *  (etx, soh, stx) the next byte will be considered as a
 *  \ref FOLLOW_UP "follow up" 0-terminated flash string. Hence by \code
  "line1 \n\0\x03\x01\x02line2 \n\0\x03\x01\x02line3 \n\0 no line4" \endcode
 *   a "pseudo flash string array for \ref outFlashTextThreadF() may be
 *   constructed that all others would see as just "line1 \\n".
 *
 *   Hint 1: Any other sequence after a terminating 0 will end this
 *   continuation.
 *
 *   Hint 2: If the \ref streamClassDescript_t "stream"  used does not provide
 *   a  \ref streamClassDescript_t "putS_P (.putS_P_func)" function correctly
 *   returning the bytes output, this feature will not work (that is stop
 *   with the first "real" string). All  \ref intro_sec "weAutSys"' streams
 *   would work.
 */
   int8_t noOfFlashStrings2out;

/** Pointer to (flash) array of (flash) strings
 *
 *  Should be \ref initOutFlashTextThread "initialised" to the string
 *  respectively array element to be output first. In the case of multiple
 *  string output this pointer will advanced.
 */
   char const * const * theFlashStrings2out;

/** Index to first character in first (current) string to be output
 *
 *  This should (and will normaly) be
 *  \ref initOutFlashTextThread "initialised" to 0.
 */
   uint16_t indToChar;
} outFlashTextThr_data_t;

/** Follow up string marker for outFlashTextThr_data_t
 *
 * \sa outFlashTextThr_data_t
 */
#define  FOLLOW_UP "\0\x03\x01\x02" //!<


/** The organisational data for a small thread hierarchy
 *
 *  This structure defines two protothreads (raw) structures and two flags.
 *  The intended use is for one main threads that can have one ore more
 *  sub-threads.
 *
 *  \sa outFlashTextThreadF
 *  \sa initOutFlashTextThread
 */
typedef struct {
   pt_t pt; //!< the (raw) protothread data structure
   uint8_t flag;  //!<  primary flag
   uint8_t flag2; //!< secondary flag
/** the (raw) protothread data structure for the secondary or sub thread */
   pt_t pt2;
   uint8_t flag3; //!< extra flag
   uint8_t flag4; //!< extra secondary flag
} hierThr_data_t;

//---------------------------------------------------------------------

/** The organisational data for a command line interpreter (CLI) thread
 *
 *  \sa registerAppCliThread
 *  \sa appCliThreadF
 *  \sa appCliThread
 *  \sa modThr_data_t
 */
struct cliThr_data_t {

/** The (raw) protothread data structure for the user CLI thread */
   pt_t pt;

/** The command number of first token
 *
 *  If found by \ref setCliLine() in the userCommands the value will be the
 *  index found there (starting at 0). If found in the \ref systemCommands
 *  the value will be that found index + \ref INDEX_OFFSET_LIST2.
 *
 *  255 means no matching token found. 254 says the command was ambiguously
 *  abbreviated. 0 is used as no command or nothing (else) to do. That
 *  schema is consistent with \ref thr_data_t "thr_data_t .flag" usage.
 */
 uint8_t commNumb;

/*   * The return code of the last schedule
 *
 *  This is intended to hold the return code, that is the command line
 *  interpreter (CLI) function's last return value.
 *
 *  \internal reserve and structure compatibility only at the moment.
 *  /
   uint8_t retC;  */

//-- End common structure ---------------------


/** The option number of first parameter or second token
 *
 *  If a second token was found by \ref setCliLine() (and set in
 *  \c paramStart, \c paramEnd) and this parameter starts with a - (minus)
 *  followed by a non digit character it will be matched against a list of
 *  standard options. If a non ambiguous match is found \c optionNumb will be
 *  set to its number and \c paramStart will be set to the next parameter
 *  (third token) if found or to to \ref optNotGivenNum "255". \c paramEnd
 *  will be set to the third token's end if given or stay at the option
 *  parameters end.
 *
 *  \ref optNotGivenNum (255) means no matching option found.
 *  \ref optAmbigousNum (254) says the option was ambiguously
 *  abbreviated. In those two cases \c paramStart and \c paramEnd still point
 *  to the (non) option parameter.
 */
 uint8_t optionNumb;

/** The (raw) protothread data structure for the system CLI thread */
    pt_t systCLIpt;

/** The streams used for output, report, reply */
 FILE * repStreams;

 char  line[LEN_OF_CLITHR_LINE+1]; //!< the input line (\ref  LEN_OF_CLITHR_LINE)
 uint8_t length;  //!< number of characters in line

 uint8_t commandStart; //!< start of command (=first token) if any
 uint8_t commandEnd;  //!< end of command (=first token) if any

 uint8_t paramStart; //!< start of first parameter (=second / third token) if any
 uint8_t paramEnd;  //!< end of first parameter (=second / third token) if any

 outFlashTextThr_data_t  outFlashTextThread; //!< for flash text output

 hierThr_data_t infoThread; //!< information threads and flags
}; // cliThr_data_t

//---------------------------------------------------------------------


/** @} */ // Doxygen addtogroup comTypes
/**
* \addtogroup modbusServer
* @{
*/

/** The configuration data (type) for a Modbus handling
 *
 */
struct modConfData_t {

/** The application specific Modbus function (pointer to)
 *
 *  The function registered here must implement the user / application
 *  specific Modbus handler as thread.
 *
 *  A NULL value here means that there is no user / application specific
 *  Modbus handler and Modbus server functions (if any) are restricted to
 *  those defined by the \ref intro_sec "weAutSys" runtime.
 *
 *  To do any process I/O or handling through Modbus user software must
 *  provide and register an appropriate function. This user function is
 *  relieved from (almost) all communication and protocol handling: it must
 *  just consume and/or deliver the respective data. (Note that 16 bit units
 *  are in wrong endianes due to Modbus standard.)
 *
 *  As provided as protothread function the user software handling of a
 *  Modbus request may block or yield. In that case the next schedule would be
 *  at the next polling the open connection. That could be 100..1000ms later.
 *  This may be appropriate in special cases and with a good explanatory
 *  statement. The recommendation is to handle the request in one stint.
 *  Anyway, the data delivered by the request have to be consumed (copied)
 *  completely in the first and the data to be send have to be given
 *  completely in the last stint.
 */
 p2ptFunM appModFun;

 /** The port of the Modbus server
  *
  *  This is and should be the well known \ref MODBUS_PORT "Modbus port".
  *  This is only provided for future use, should it be decided to make the
  *  Modbus server port configurable.
  */
 uint16_t port;

 /** The unit number of the Modbus server
  *
  *  If this server is to check the unit number in requests this is the
  *  only one accepted. Other wise the request's unit number and this value
  *  will be ignored (i.e. all will be accepted).
  */
 uint8_t unit;

 /** Configuration options of the Modbus server
   *
   *  Bit 0 (0x01) : if set check request's unit number
   *                 against \c .unit <br />
   *  Bit 1..7     : future use <br />
   */
  uint8_t options;

}; //  modConfData_t

/** The (start of a) Modbus TCP/IP telegram
 *
 *  The first seven bytes are the so called MBAB header.
 *
 *  The next 13 bytes are the (start / part of) the the so called PDU, the
 *  first byte of which always is the function code (1..127 / 0x01 0x7F) or
 *  error return code (129..255 / 0x81..0xFF).
 *
 *  \sa formModTelegr
 */
struct modTelegr_t {

/** Transaction identifier
 *
 *  Set by client &mdash; to be responded unchanged.
 */
   ucnt16_t transId;

/** Protocol identifier
 *
 *  Must be set 0x0000 by client (= Modbus) &mdash; and be so in response.
 *
 *  Remark: This word seems totally redundant in Modbus TCP/IP as the port
 *  (lport) identifies the \ref MODBUS_PORT "protocol".
 */
   ucnt16_t protId;

/** Bytes in the telegram to followProtocol identifier
 *
 *  This is the byte count from and including the unit identifier byte.
 *  Is set by the client according to request length as well as by the server
 *  according to respose length.
 *
 *  Note 1: This as all 16 bit (word) values with Modbus is in wrong &mdash;
 *  big endian &mdash; byte order.
 *
 *  Note 2: This word is totally redundant in Modbus TCP/IP as it always must
 *  be telegram's length - 6.
 */
   ucnt16_t byToFol;

/** The unit number
 *
 *  Is set by client and must responded unchanged.
 *
 *  An (user) implementation may decide to handle multiple units or to have
 *  an own special unit number and check if this number in the request is its
 *  own. In all else cases this value will be ignored.
 */
   uint8_t unitNumb;

/** The code of the function to perform
 *
 *  Is set by client request and usually copied as
 *  \ref modThr_data_t "modThr_data_t.fcFlag" for handler function / thread
 *  flag usage.
 *
 *  The value must responded unchanged in error free responses or with
 *  bit 7 (0x80) set as error response.
 */
   uint8_t fCode;

 /** The (start of the) PDU content
  *
  *  If pairs of those 12 bytes are to be interpreted as 16 bit / word
  *  values they are in wronr byte order.
  */
   ucnt16_t pduCont[6];

}; // modTelegr_t

/** The organisational data for a Modbus handler thread
 *
 *  The similarity of structure of modThr_data_t data and
 *  \ref  cliThr_data_t "those" for \ref  cliThr_data_t CLI is not by chance.
 */
struct modThr_data_t {

/** The (raw) protothread data structure for the user modThr_data_t thread */
   pt_t pt;

/** The outstanding function to perform
 *
 *  It is an accepted function code (FC) or 0. 0 is used as no code or
 *  nothing to do.  That appointment is consistent with
 *  \ref thr_data_t "thr_data_t .flag" usage.
 */
 uint8_t fcFlag;

/** The return code of the last schedule
 *
 *  This is intended to hold the return code, that is the Modbus handler
 *  function's last return value.
 *
 *  \internal reserve and structure compatibility only at the moment.
 */
   uint8_t retC;

/** The system's Modbus configuration (copy of as of time of request)
 *
 */
   struct modConfData_t config;

//-- End common structure - within config --------------


/** The complete length of the incoming Modbus request (telegram) */
  uint16_t gotLen;

/** A copy of (the start of the) incoming request */
  struct modTelegr_t reqModTelegr;

/** Poll count
 *
 *  Value 0 means first stint for handler function / tread; i.e. new data.
 *  Value 1 means second stint or first connection poll, and so on.
 */
   uint8_t pollCnt;

/** Data model for in and / or out data
 *
 *  The values provided are \ref  DISC_INP_BYMAPD "Discrete inputs",
 *  \ref COIL_SINGLE "Coil", \ref COILS_BYMAPD "Coils",
 *  \ref INP_REGISTERS "Input registers", \ref HLD_MASK  "Mask single Holding"
 *  and \ref HLD_REGISTERS "Holding registers". It will be set according to
 *  function code before the user / application Modbus handler is called.
 */
   uint8_t dataModel;

/** Error code
 *
 *  To be set (!=0) by the handler if the requested function cannot be
 *  performed.
 */
   uint8_t errorCode;

// -- incoming data (POI) for the handler function

/** The length of the request's incoming data as byte count */
   uint8_t inDataByteCnt;

 /** The (logic) start or target address for the request's incoming data
  *
  *  This value will be provided in correct byte order.
  */
   ucnt16_t inDatTarg;

/** Incoming data (pointer to)
 *
 *  This pointer is only valid if \c .inDatlen is not 0 and only in the
 *  first stint of a handling protothread function. For the type and
 *  interpretation of the data see primarily the function code.
 */
   uint8_t * inData;

 // -- outgoing data (PII) for the handler function

/** The length of the responses outgoing data as byte count
 *
 *  This field is set to expected value. It may be shortened by the
 *  handler thread / function if that decides to deliver less data that
 *  requested.
 */
   uint8_t outDataByteCnt;

/** The (logic) start or source address for the response's outgoing data
 *
 *  This value will be provided in correct byte order.
 */
   ucnt16_t outDatSou;

 /** Outgoing data (pointer to)
  *
  *  This pointer is only valid if \c .outDataBytCnt is not 0 and only in the
  *  last stint of a handling protothread function.
  */
    uint8_t * outData;

 /** Working variables for the Modbus handling threads
  *
  *  This must be set according to the bytes copied to \c .outData (in the
  *  last stint) if any.
  */
     ucnt32_t wrkV;

}; //  modThr_data_t

/** @} */ // Doxygen addtogroup modbusServer
/**
* \addtogroup comTypes
* @{
*/

//------- threading (common) structures      ---------------------------------

/** State data for a thread (minimal)
 *
 *  It is strongly recommended that threads use this structure for their
 *  essential state. The thread function's (the behaviour) recommended
 *  signature takes (only) a pointer to such structure as parameter.
 *
 *  Software using this schema is supported by some handling functions and
 *  macros provided. If more state info is required the extended type
 *  \ref thr_data_t should be used; it can be cast back to this common
 *  smaller type \ref mThr_data_t to use the appropriate handling functions.
 *
 *  \sa #resetAppThread
 *  \sa #scheduleAppThread
 *  \sa #scheduleRegdAppThread
 *  \sa #scheduleFlgdAppThread
 *  \sa #scheduleYldAppThread
 *  \sa registerAppThread(struct mThr_data_t *, p2ptFun)
 *  \sa initSetAppThread(struct mThr_data_t *, p2ptFun)
 */
struct mThr_data_t {
/** The (raw) protothread data structure  */
   pt_t pt;

/** A flag byte for the threads specific usage
 *
 *  This is intended to hold (additionally to \c pt) the primary thread state.
 *  The semantic is defined by the thread function (the behaviour). <br />
 *
 *  Insofar the thread implementation (i.e. the thread function code) is
 *  totally free with respect to this \c flag's semantic. The only fixed
 *  common convention is <br />
 *  &nbsp; 0 : "no work to do" for the thread <br />
 *  This convention should be strictly observed, as \ref intro_sec "weAutSys" may not schedule
 *  a thread on \c !flag.
 *
 *  \c flag is used as workload count for cyclic threads respectively
 *  periodic tasks in \ref intro_sec "weAutSys"; that means something like <br />
 *  &nbsp; 0 : "do no periodic work even if scheduled" <br />
 *  &nbsp; 1 : "do normal periodic work" <br />
 *  &nbsp; 2.. means cycle overrun "do the work of 2.. periods" <br />
 *
 *  As said above, the value 0 meaning "nothing to do" is the only rule
 *  never to be violated.
 */
  uint8_t flag;

/** The return code of the last schedule
 *
 *  This is intended as a secondary status flag. The recommended (and
 *  supported) use is to hold the thread function's last return value.
 *
 *  Hint: Some supporting functions and macros use this \c retC to treat a
 *  yielding thread in a special way: it might be scheduled with flag == 0 for
 *  doing rest work but a blocking thread will not. Their criterion is just
 *  \c retC \c == \ref PT_YIELDED
 */
   uint8_t retC;

/** The thread's function (pointer to)
 *
 *  The function must implement a thread (behaviour) according to all
 *  Protothreads and \ref intro_sec "weAutSys" rules therefore. This pointer
 *  is binding this thread state to a thread function.
 *
 *  Hint / rationale: It is quite possible to have one
 *  \ref p2ptFun "thread function" handle multiple
 *  \ref mThr_data_t "thread objects" implementing a common behaviour. This
 *  function must keep all thread state in the
 *  \ref mThr_data_t "thread structure". But to have this state data never in
 *  in any other (static) variables is good practice anyway.
 *
 *  The other way round is never feasible: no living Protothread can be
 *  handled by more than one \ref p2ptFun "thread function" at the same
 *  time. <br />
 *  Hence it is quite logical to (optionally) register this one / current
 *  \ref p2ptFun "thread function" here.
 *
 *  This being NULL may hence mean this \ref mThr_data_t "thread " having
 *  no registered function respectively behaviour.
 *
 *  Hint 2: If the Protothread function is statically fixed for a set of
 *  threads (and none of the  above mentioned scheduling macros is used on
 *  them) this \c fun pointer may be used for other purposes.
 */
   p2ptFun fun;

};  // mThr_data_t


/** State data for a thread (universal variable type)
 *
 *  It is strongly recommended that threads with states larger than
 *  fitting in \ref mThr_data_t should use this structure (extending
 *  \ref mThr_data_t) for their state. The thread function's (the behaviour)
 *  recommended signature takes (only) a pointer to such structure as
 *  parameter.
 *
 *  Software using this schema is supported by some handling functions and
 *  macros provided.
 *
 *  \note The first four elements (pt, flag, retC, fun) are exactly the
 *  structure \ref mThr_data_t.
 */
struct thr_data_t {
/** The (raw) protothread data structure
 *
 *  This is the same as \c .pt in \ref mThr_data_t.
 */
   pt_t pt;

/** A flag byte for the threads specific usage
 *
 *  This is the same as \c .flag in \ref mThr_data_t.
 */
  uint8_t flag;

/** The return code of the last schedule
 *
 *  This is the same as \c retC in \ref mThr_data_t.
 */
   uint8_t retC;

/** The thread's function (pointer to)
 *
 *  This is the same as \c .fun in \ref mThr_data_t. For compatibility with
 *  the bequeather even the type is the same, even though in this larger
 *  structure the function pointer put here must be of type \ref  p2ptFunC.
 */
   p2ptFun fun;

/** The thread's respectively thread functions state data
 *
 *  It is strongly recommended to use no other external or static data for
 *  whatever else thread state data but to keep it here in this union. The
 *  char [] variant may be cast to any application defined structure of
 *  \ref EXTRA_THR_ST_SZ "fitting length".
 */
   union {
      char stateA[EXTRA_THR_ST_SZ]; //!< Byte / char array (castable)
      struct cliThr_data_t  cliThrData; //!< Commandline interpreter structure variant
      struct modThr_data_t modThrData; //!< Modbus handler variant
   };
};  // thr_data_t


/** @} */ // Doxygen addtogroup comTypes
/** @} */ // Doxygen addtogroup appLaySup

#endif /* COMMON_H */
/** @} */ // Doxygen addtogroup weAutSys
