/**
 *  \file uip_intf.c
 *
 *  weAutSys  implementation of uIP  system interface
 *
 *  This is system software and must not be modified for user or
 *  application programs.
 *
 * This file is part of weAutSys on weAut_01 &nbsp;
 *          <<a href="http://weinert-automation.de">weinert-automation.de</a>>
 *
 * &nbsp; \ref copyLicNote " Copyright &copy; " 2011 Albrecht Weinert, Bochum
 * \author  Albrecht Weinert  &nbsp; <<a href="http://a-weinert.de">a-weinert.de</a>>
 * 2
 * 25.01.2017
 */
//  SVN last change by A. Weinert

#include "we-aut_sys/common.h"
#include "we-aut_sys/system.h"
#include "we-aut_sys/syst_threads.h"

#include "we-aut_sys/network.h"
#include "we-aut_sys/ntp.h"
#include "we-aut_sys/log_streams.h"

#include "uip/uip.h"
#include "uip/apps/dhcpc.h"
#include "uip/apps/resolv.h"

#include <string.h>

// extern char uipAppResolFMes[];

void uip_log(char * s){ puts(s); }

#ifdef   __DHCPC_H__  // quite nonsense, as defined by above include


#if  DEBUG_DHCP
//  #include "we-aut_sys/log_streams.h"
//  #include "we-aut_sys/uart0.h"

INFLASH(char const dhcpOptiMes[]) = "DHCP option: ";
//                             0123456789.123
INFLASH(char const dhcpConfiguredMes[]) = "DHCP configured: ";
//                                   0123456 89.1234567
#endif


/* DHCP option received
 *
 *  This (callback) signals an option send by the DHCP server.
 *  This implementation uses   curIpConf  directly to store all
 *  relevant DHCP answers. It might be better to  to have separate
 *  dhcpIpConf  for that  and copy it at the and (dhcpc_configured())
 *  to curIpConf, initialised by defaultConfData.ipConf
 *  \param dhcpOption the option (one of the options) received
 *  \
 */
void dhcpcGotOption(const struct dhcpOption_t *dhcpOption){
   const char * const vP = dhcpOption->value;
 //  vP = dhcpOption->value;
#if DEBUG_DHCP
   bufLogDecHex_P(dhcpOptiMes, dhcpOption->code,
                        //   *((uint32_t *)dhcpOption->value));
                          *((uint32_t *)vP) );

#endif
   switch(dhcpOption->code) {
   case DHCP_OPTION_SUBNET_MASK:
     uip_ipaddr_copy(curIpConf.netMask,vP); // dhcpOption->value);
     break;
   case DHCP_OPTION_ROUTER:
      uip_ipaddr_copy(curIpConf.defaultRouter, vP);
     break;
   case DHCP_OPTION_DNS_SERVER:
     uip_ipaddr_copy(curIpConf.dnsAddr, vP);
     curIpConf.setFlags |= DNS1_MSK;
     if (dhcpOption->length &0xF8)  { // >= 8
        uip_ipaddr_copy(curIpConf.dns2Addr, vP + 4);
        curIpConf.setFlags |= DNS2_MSK;
     } else {
        curIpConf.setFlags &= ~DNS2_MSK;
     }
     break;
   case DHCP_OPTION_NTP_SERVERS:
     uip_ipaddr_copy(curIpConf.ntpAddr, vP);
     curIpConf.setFlags |= NTP1_MSK;
     if (dhcpOption->length &0xF8)  { // >= 8
        uip_ipaddr_copy(curIpConf.ntp2Addr, vP + 4);
        curIpConf.setFlags |= NTP2_MSK;
     } else {
        curIpConf.setFlags &= ~NTP2_MSK;
     }
     break;

   case DHCP_OPTION_TIME_OFFSET:
      setZoneOffsetSec(__builtin_bswap32(*((uint32_t *)vP)));
      break;

   case DHCP_OPTION_LEASE_TIME:
     curIpConf.leaseTime = __builtin_bswap32(*((uint32_t *)vP));
     break;

   // DHCP_OPTION_MSG_TYPE, DHCP_OPTION_SERVER_ID and
   // DHCP_OPTION_RENEW_TIME  are handled elsewhere (parse_options())

   // case DHCP_OPTION_MSG_TYPE:  break; // may be used for for
   // case DHCP_OPTION_END: break;      //  internal state machine
   } // switch
} // dhcpcGotOption

/* DHCP success
 *
 *  This (callback) signals a successful walk through the DHCP protocol
 *  steps ended by acknowledge.
 *  \param respType at present always \ref DHCPACK; (no callback in other
 *                  cases yet)
 *  \param ipAddr   the assigned / leased IP address (in case of success)
 */
void dhcpc_configured(uint8_t respType, const uint16_t ipAddr[]){
  uip_ipaddr_copy(curIpConf.ipAddr, ipAddr);
  curIpConf.setTime = secClock();
  curIpConf.setFlags |= DHCP_MSK;
  uipSetAddresses(&curIpConf);

#if DEBUG_DHCP
   bufLogDec_P(dhcpConfiguredMes, respType); //
#endif
   ntpReset();
   ntpInit(0); // does check curIpConf.setFlags & NTP1_MSK itself
} // dhcpc_configured

#endif /* __DHCPC_H__ */

//                                   0123456 89.123456789 .
INFLASH(char uipAppResolFMes[]  ) = "uIP resolv_f : ... \n"; //!< resolve report

/*  Callback function which is called when a hostname is found
 *
 *  This function must be implemented by the module that uses the DNS
 *  resolver. It is called when a hostname is found, or when a hostname
 *  was not found.
 *  This does nothing but debug message.
 *
 *  \param  name   A pointer to the name that was looked up.
 *  \param  ipaddr A pointer to a 4-byte array containing the IP address
 *          of the hostname, or NULL if the hostname could not be found.
 */
void resolv_found(char * name, uint16_t * ipaddr){
   bufLogTxt_P(uipAppResolFMes);
   bufLogTxt(name, 63);
   //            0123456789.123456789z1 2
   char tmp[] = " = null ??           \n";

   if (ipaddr) {
       * formIpAdd(&tmp[3], ipaddr) = ' ';
   }
   bufLogTxt(tmp, 22);
} // resolv_found(char *, uint16_t *)

